import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOptionsWhere, Repository } from 'typeorm';
import { UpdateNotificationsSettingsRequestDto } from '~/customer/dtos/request';
import { Customer } from '~/customer/entities';
import { User, UserNotificationSettings } from '../entities';

@Injectable()
export class UserRepository {
  constructor(@InjectRepository(User) private readonly userRepository: Repository<User>) {}

  createUnverifiedUser(data: Partial<User>) {
    return this.userRepository.save(
      this.userRepository.create({
        phoneNumber: data.phoneNumber,
        countryCode: data.countryCode,
        roles: data.roles,
        notificationSettings: UserNotificationSettings.create(),
      }),
    );
  }

  findOne(where: FindOptionsWhere<User>) {
    return this.userRepository.findOne({ where });
  }

  updateNotificationSettings(user: User, body: UpdateNotificationsSettingsRequestDto) {
    user.notificationSettings = UserNotificationSettings.create({ ...user.notificationSettings, ...body });
    return this.userRepository.save(user);
  }

  verifyUserAndCreateCustomer(user: User) {
    user.customer = Customer.create({ ...user.customer, id: user.id, isGuardian: true });

    return this.userRepository.save(user);
  }

  update(userId: string, data: Partial<User>) {
    return this.userRepository.update(userId, data);
  }
}
